@@ -5,6 +5,9 @@
  
 /* Define if zlib should be used */
 #undef WITH_ZLIB
+
+/* Define if mhash should be used */
+#undef WITH_MHASH
  
 /* Define if zlib.h is present */
 #undef HAVE_ZLIB_H
@@ -12,8 +15,8 @@
 /* syslog() available? */
 #undef HAVE_SYSLOG
 
-/* Use bundled regexps do NOT change */
-#define REGEX
+/* Defined if we use bundled regexps */
+#undef REGEX
 
 /* Default configuration file */
 #define CONFIG_FILE "./aide.conf" 
@@ -24,8 +27,12 @@
 #undef HAVE_U16_TYPEDEF
 #undef HAVE_U32_TYPEDEF
 
+#undef BIG_ENDIAN_HOST
+#undef LITTLE_ENDIAN_HOST
+
 /* AIDE compile options */
 #undef AIDECOMPILEOPTIONS
 
 #undef HAVE_readdir
 #undef HAVE_readdir_r
+#undef HAVE_mmap
