@@ -42,9 +42,11 @@ typedef struct url_t {
 
 
 typedef enum { db_filename, db_perm, db_uid, db_gid,
-	       db_size, db_atime, db_ctime, db_mtime, db_inode, 
-	       db_lnkcount, db_md5, db_sha1, db_rmd160, db_tiger, 
-	       db_unknown } DB_FIELD;
+	       db_size, db_atime, db_ctime, db_mtime, db_inode,
+	       db_bcount, db_lnkcount, db_md5, db_sha1, db_rmd160,
+	       db_tiger, db_crc32, db_haval, db_rmd128, db_snefru,
+	       db_gost, db_crc32b, db_unknown } DB_FIELD;
+
 /* db_unknown must be last because it is used to determine size of
    DB_FILED */
 
@@ -59,12 +61,20 @@ typedef enum { db_filename, db_perm, db_uid, db_gid,
 #define DB_CTIME    (1<<7)
 #define DB_MTIME    (1<<8)
 #define DB_INODE    (1<<9)
-#define DB_LNKCOUNT (1<<10)
-#define DB_MD5      (1<<11)
-#define DB_SHA1     (1<<12)
-#define DB_RMD160   (1<<13)
-#define DB_TIGER    (1<<14)
-
+#define DB_BCOUNT   (1<<10)
+#define DB_LNKCOUNT (1<<11)
+#define DB_MD5      (1<<12)
+#define DB_SHA1     (1<<13)
+#define DB_RMD160   (1<<14)
+#define DB_TIGER    (1<<15)
+#ifdef WITH_MHASH
+#define DB_CRC32    (1<<16)
+#define DB_HAVAL    (1<<17)
+#define DB_RMD128   (1<<18)
+#define DB_SNEFRU   (1<<19)
+#define DB_GOST     (1<<20)
+#define DB_CRC32B   (1<<21)
+#endif
 
 /* TIMEBUFSIZE should be exactly ceil(sizeof(time_t)*8*ln(2)/ln(10))
  * Now it is ceil(sizeof(time_t)*2.5)
@@ -110,6 +120,9 @@ typedef struct db_config {
   /* What are we supposed to do */
   int action;
 
+  /* Should we catch errors from mmapping */
+  int catch_mmap;
+
 } db_config;
 
 typedef struct db_line {
@@ -117,8 +130,17 @@ typedef struct db_line {
   byte* sha1;
   byte* rmd160;
   byte* tiger;
+#ifdef WITH_MHASH
+  byte* crc32;
+  byte* haval;
+  byte* rmd128;
+  byte* snefru;
+  byte* gost;
+  byte* crc32b;
+#endif
 
   mode_t perm;
+  mode_t perm_o; /* Permission for tree traverse */
   uid_t uid;
   gid_t gid;
   time_t atime;
@@ -128,6 +150,8 @@ typedef struct db_line {
   nlink_t nlink;
   
   off_t size;
+  off_t size_o; /* ... */
+  unsigned long bcount;
   char* filename;
 
   /* Attribuutes .... */
