@@ -1,6 +1,6 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999 Rami Lehti
+ * Copyright (C) 1999 Rami Lehti, Pablo Virolainen
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,13 +20,20 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <signal.h>
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
 #include "aide.h"
+
+#ifdef WITH_MHASH
+#include <mhash.h>
+#endif
+
 #include "cipher.h"
+
 #include "commandconf.h"
 #include "compare_db.h"
 #include "db_config.h"
@@ -36,6 +43,7 @@
 #include "gen_list.h"
 #include "getopt.h"
 #include "list.h"
+#include "util.h"
 
 db_config* conf;
 
@@ -234,6 +242,7 @@ void setdefaults_before_config()
   conf->db_out_url=NULL;
   conf->db_out=NULL;
   conf->action=0;
+  conf->catch_mmap=0;
 
   conf->selrxlst=NULL;
   conf->equrxlst=NULL;
@@ -241,7 +250,7 @@ void setdefaults_before_config()
 
   conf->defsyms=NULL;
   conf->groupsyms=NULL;
-
+  
   do_groupdef("p",DB_PERM);
   do_groupdef("i",DB_INODE);
   do_groupdef("n",DB_LNKCOUNT);
@@ -249,6 +258,7 @@ void setdefaults_before_config()
   do_groupdef("g",DB_GID);
   do_groupdef("s",DB_SIZE);
   do_groupdef("S",DB_SIZEG);
+  do_groupdef("b",DB_BCOUNT);
   do_groupdef("m",DB_MTIME);
   do_groupdef("c",DB_CTIME);
   do_groupdef("a",DB_ATIME);
@@ -256,6 +266,18 @@ void setdefaults_before_config()
   do_groupdef("tiger",DB_TIGER);
   do_groupdef("sha1",DB_SHA1);
   do_groupdef("rmd160",DB_RMD160);
+#ifdef WITH_MHASH
+  do_groupdef("rmd128",DB_RMD128);
+  do_groupdef("crc32",DB_CRC32);
+  /*
+    crc32b is same than crc32
+    do_groupdef("crc32b",DB_CRC32B);
+  */
+  do_groupdef("haval",DB_HAVAL);
+  do_groupdef("gost",DB_GOST);
+  do_groupdef("snefru",DB_SNEFRU); 
+#endif
+
   do_groupdef("R",DB_PERM|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_SIZE|
 	      DB_MTIME|DB_CTIME|DB_MD5);
   do_groupdef("L",DB_PERM|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID);
@@ -271,14 +293,14 @@ void setdefaults_after_config()
     url_t* u=NULL;
     u=(url_t*)malloc(sizeof(url_t));
     u->type=url_file;
-    u->value="aide.db";
+    u->value="/etc/aide.db";
     conf->db_in_url=u;
   }
   if(conf->db_out_url==NULL){
     url_t* u=NULL;
     u=(url_t*)malloc(sizeof(url_t));
     u->type=url_file;
-    u->value="aide.db.new";
+    u->value="/etc/aide.db.new";
     conf->db_out_url=u;
   }
   if(conf->report_url==NULL){
@@ -296,12 +318,15 @@ void setdefaults_after_config()
   }
 }
 
+
 int main(int argc,char**argv)
 {
   list* newlst=NULL;
   list* l=NULL;
   int errorno=0;
 
+  init_sighandler();
+
   setdefaults_before_config();
 
   if(argc<=1){
@@ -334,15 +359,19 @@ int main(int argc,char**argv)
     }
   };
   newlst=gen_list(conf->selrxlst,conf->negrxlst,conf->equrxlst);
-  newlst=do_md(newlst);
-
+  
   if(conf->action&DO_INIT){
-    if(db_init(0)==RETFAIL)
+    if(db_init(conf,0)==RETFAIL)
+      exit(IO_ERROR);
+    if(db_writespec(conf)==RETFAIL){
+      error(0,"Error while writing database. Exiting..\n");
       exit(IO_ERROR);
-    if(db_writespec()==RETFAIL){
-	error(0,"Error while writing database. Exiting..\n");
-	exit(IO_ERROR);
     }
+  }
+  
+  newlst=do_md(newlst,conf);
+  
+  if(conf->action&DO_INIT){
     for(l=newlst;l;l=l->next){
       if(db_writeline((db_line*)l->data,conf)==RETFAIL){
 	error(0,"Error while writing database. Exiting..\n");
@@ -352,10 +381,12 @@ int main(int argc,char**argv)
   }
   
   if(conf->action&DO_COMPARE){
-    if(db_init(1)==RETFAIL)
+    if(db_init(conf,1)==RETFAIL)
       exit(IO_ERROR);
-    compare_db(newlst);
+    compare_db(newlst,conf);
   }
-
+  
+  db_close(conf);
+  
   return RETOK;
 }
