@@ -81,9 +81,11 @@ FAIL, FAIL, FAIL, FAIL, FAIL, FAIL, FAIL, FAIL
 };
 
 /* Returns NULL on error */
+/* FIXME Possible buffer overflow on outputs larger than B64_BUF */
 char* encode_base64(byte* src,size_t ssize)
 {
   char* outbuf;
+  char* retbuf;
   int pos;
   int i, l, left;
   unsigned long triple;
@@ -157,13 +159,21 @@ char* encode_base64(byte* src,size_t ssize)
 	  }
       inb++;
   }
+  
+  /* outbuf is not completely used so we use retbuf */
+  retbuf=(char*)malloc(sizeof(char)*(pos+1));
+  memcpy(retbuf,outbuf,pos);
+  retbuf[pos]='\0';
+  free(outbuf);
 
-  return outbuf;
+  return retbuf;
 }
 
+/* FIXME Possible buffer overflow on outputs larger than B64_BUF */
 byte* decode_base64(char* src,size_t ssize)
 {
   byte* outbuf;
+  byte* retbuf;
   char* inb;
   int i;
   int l;
@@ -231,6 +241,13 @@ byte* decode_base64(char* src,size_t ssize)
 	}
       inb++;
     }
-  return outbuf;
+  
+  retbuf=(byte*)malloc(sizeof(byte)*(pos+1));
+  memcpy(retbuf,outbuf,pos);
+  retbuf[pos]='\0';
+  
+  free(outbuf);
+
+  return retbuf;
 }
 
