@@ -33,11 +33,12 @@ FILE* be_init(int inout,url_t* u)
 
   switch (u->type) {
   case url_file : {
+    error(200,"Opening file \"%s\" for %s\n",u->value,inout?"r":"w+");
     fh=fopen(u->value,inout?"r":"w+");
-    if(fh==NULL)
+    if(fh==NULL){
       error(0,"Couldn't open file %s for %s",u->value,
 	    inout?"reading\n":"writing\n");
-
+    }
     return fh;
   }
   case url_stdout : {
@@ -66,4 +67,6 @@ FILE* be_init(int inout,url_t* u)
   }    
   }
   /* Not reached */
+  return NULL;
+
 }
