@@ -32,6 +32,7 @@
 #include "symboltable.h"
 #include "util.h"
 
+
 #define BUFSIZE 4096
 
 #ifndef MAXHOSTNAMELEN
@@ -157,6 +158,9 @@ void update_db_out_order(int a)
   if((a&DB_PERM) && (check_dboo(db_perm)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_perm;
   }
+  if((a&DB_BCOUNT) && (check_dboo(db_bcount)!=RETFAIL)){
+    conf->db_out_order[conf->db_out_size++]=db_bcount;
+  }
   if((a&DB_UID) && (check_dboo(db_uid)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_uid;
   }
@@ -426,10 +430,12 @@ void do_dbindef(char* val)
     /* FIXME Check the URL if you add support for databases that cannot be 
      * both input and output urls */
     if(u==NULL||u->type==url_unknown||u->type==url_stdout
-       ||u->type==url_stderr)
-      error(0,"Unsupported input URL-type:%s",val);
-    else
+       ||u->type==url_stderr) {
+      error(0,"Unsupported input URL-type:%s\n",val);
+    }
+    else {
       conf->db_in_url=u;
+    }
   }
 
   free(val);
@@ -439,14 +445,21 @@ void do_dboutdef(char* val)
 {
   url_t* u=NULL;
 
+  error(200,"Setting output database \"%s\"\n",val);
+
   if(conf->db_out_url==NULL){
     u=parse_url(val);
     /* FIXME Check the URL if you add support for databases that cannot be 
      * both input and output urls */
-    if(u==NULL||u->type==url_unknown||u->type==url_stdin)
-      error(0,"Unsupported output URL-type:%s",val);
-    else
+    if(u==NULL||u->type==url_unknown||u->type==url_stdin){
+      error(0,"Unsupported output URL-type:%s\n",val);
+    }
+    else{
       conf->db_out_url=u;
+      error(200,"Output database set to \"%s\" \"%s\"\n",val,u->value);
+    }
+  } else {
+    error(200,"Output database already set\n");
   }
 
   free(val);
@@ -461,7 +474,7 @@ void do_repurldef(char* val)
   /* FIXME Check the URL if you add support for databases that cannot be 
    * both input and output urls */
   if(u==NULL||u->type==url_unknown||u->type==url_stdin){
-    error(0,"Unsupported output URL-type:%s",val);
+    error(0,"Unsupported output URL-type:%s\n",val);
   } else {
     error_init(u);
   }
