@@ -26,7 +26,11 @@
 #include "db_config.h"
 #include "error.h"
 #include "be.h"
-#include "cipher.h"
+
+#ifdef WITH_MHASH
+#include <mhash.h>
+#endif
+
 #include "base64.h"
 #include "util.h"
 #include "aide.h"
@@ -36,20 +40,28 @@ long readint(char* s,char* err);
 long readoct(char* s,char* err);
 
 time_t base64totime_t(char*);
-
-int db_init(int inout)
+int db_init(db_config* conf,int inout)
 {
+  
+  error(200,"db_init %i\n",inout);
+
   if(inout==1){
     conf->db_in=be_init(1,conf->db_in_url);
-    if(conf->db_in==NULL)
+    if(conf->db_in==NULL) {
       return RETFAIL;
+    }
+    return RETOK;
   }
   if(inout==0){
     conf->db_out=be_init(0,conf->db_out_url);
-    if(conf->db_out==NULL)
+    if(conf->db_out==NULL){
+      error(200,"db_out is null\n");
       return RETFAIL;
+    }
+    error(200,"db_out is nonnull %s\n",conf->db_out_url->value);
+    return RETOK;
   }
-  return RETOK;
+  return RETFAIL;
 }
 
 db_line* db_readline(db_config* conf){
@@ -68,9 +80,9 @@ db_line* db_readline(db_config* conf){
 	s=db_char2line(ss,conf);
 
 	for(i=0;i<conf->db_in_size;i++){
-	  if(ss[i]!=NULL){
-	    free(ss[i]);
-	    ss[i]=NULL;
+	  if(ss[conf->db_in_order[i]]!=NULL){
+	    free(ss[conf->db_in_order[i]]);
+	    ss[conf->db_in_order[i]]=NULL;
 	  }
 	}
 	free(ss);
@@ -108,6 +120,14 @@ db_line* db_char2line(char** ss,db_config* conf){
   line->sha1=NULL;
   line->rmd160=NULL;
   line->tiger=NULL;
+#ifdef WITH_MHASH
+  line->crc32=NULL;
+  line->crc32b=NULL;
+  line->haval=NULL;
+  line->rmd128=NULL;
+  line->snefru=NULL;
+  line->gost=NULL;
+#endif
   line->perm=0;
   line->uid=0;
   line->gid=0;
@@ -116,6 +136,7 @@ db_line* db_char2line(char** ss,db_config* conf){
   line->mtime=0;
   line->inode=0;
   line->nlink=0;
+  line->bcount=0;
   line->size=0;
   line->filename=NULL;
   line->attr=0;
@@ -136,6 +157,10 @@ db_line* db_char2line(char** ss,db_config* conf){
       line->mtime=base64totime_t(ss[conf->db_in_order[i]]);
       break;
     }
+    case db_bcount : {
+      line->bcount=readint(ss[conf->db_in_order[i]],"bcount");
+      break;
+    }
     case db_atime : {
       line->atime=base64totime_t(ss[conf->db_in_order[i]]);
       break;
@@ -250,7 +275,7 @@ long readoct(char* s,char* err){
 }
 
 
-int db_writespec()
+int db_writespec(db_config* conf)
 {
   switch (conf->db_out_url->type) {
   case url_stdout:
@@ -258,7 +283,7 @@ int db_writespec()
   case url_fd:
   case url_file: {
     if(conf->db_out!=NULL){
-      if(db_writespec_file()==RETOK){
+      if(db_writespec_file(conf)==RETOK){
 	return RETOK;
       }
     }
@@ -301,7 +326,7 @@ int db_writeline(db_line* line,db_config* conf){
 }
 
 /* Please FIXME */
-int close_db(void)
+int db_close(db_config* conf)
 {
-  return 0;
+  return RETFAIL;
 }
