@@ -24,24 +24,31 @@
 
 #include "types.h"
 #include "base64.h"
-#include "cipher.h"
 #include "db_file.h"
 #include "conf_yacc.h"
 #include "util.h"
 #include "aide.h"
 
+#ifdef WITH_MHASH
+#include <mhash.h>
+#endif
+
+#include "cipher.h"
+
 extern int db_scan(); /* Rumaa.... */
 extern char* dbtext; /* Todella rumaa... */
 extern long db_lineno;
 
 const static char* db_names[] = {
-  "name","perm","uid","gid","size","atime","ctime","mtime","inode","count",
-  "md5","sha1","rmd160","tiger", "unknown" };
+  "name", "bcount", "perm", "uid", "gid", "size", "atime", "ctime",
+  "mtime", "inode", "count", "md5", "sha1", "rmd160", "tiger",
+  "crc32", "haval", "rmd128","snefru","gost","crc32b", "unknown" }; 
 
 const static int db_value[] = {
-  db_filename, db_perm, db_uid, db_gid,
-  db_size, db_atime, db_ctime, db_mtime, db_inode, 
-  db_lnkcount, db_md5, db_sha1, db_rmd160, db_tiger, 
+  db_filename, db_bcount, db_perm, db_uid, db_gid, db_size, db_atime,
+  db_ctime, db_mtime, db_inode, db_lnkcount, db_md5, db_sha1,
+  db_rmd160, db_tiger, db_crc32, db_haval, db_rmd128, db_snefru,
+  db_gost, db_crc32b,
   db_unknown };
 
 
@@ -110,11 +117,17 @@ char** db_readline_file(db_config* conf){
   int i=0;
   int r;
   int a=0;
+  int token=0;
   
   if (conf->db_in_size==0) {
     db_buff(conf->db_in);
     
-    if (db_scan()!=TDBSPEC) {
+    token=db_scan();
+    while(token==TUNKNOWN){
+      token=db_scan();
+    }
+
+    if (token!=TDBSPEC) {
       /*
        * error.. must be a @@dbspec line
        */
@@ -307,6 +320,7 @@ int db_write_byte_base64(byte*data,size_t len,FILE* file,int i )
     return fprintf(file,"0");
   }
   return 0;
+
 }
 
 int db_write_time_base64(time_t i,FILE* file,int a)
@@ -347,34 +361,38 @@ int db_writeoct(long i, FILE* file,int a)
   
 }
 
-int db_writespec_file()
+int db_writespec_file(db_config* conf)
 {
   int i=0;
   int j=0;
   int retval=1;
 
-  retval=fprintf(conf->db_out,"# This file was generated by Aide, version %s\n",AIDEVERSION);
-  if(retval==0)
+  retval=fprintf(conf->db_out,
+		 "# This file was generated by Aide, version %s\n",
+		 AIDEVERSION);
+  if(retval==0){
     return RETFAIL;
+  }
 
   retval=fprintf(conf->db_out,"@@db_spec ");
-  if(retval==0)
+  if(retval==0){
     return RETFAIL;
-  
+  }
   for(i=0;i<conf->db_out_size;i++){
     for(j=0;j<db_unknown;j++){
       if(db_value[j]==conf->db_out_order[i]){
 	retval=fprintf(conf->db_out,"%s ",db_names[j]);
-	if(retval==0)
+	if(retval==0){
 	  return RETFAIL;
+	}
 	break;
       }
     }
   }
   retval=fprintf(conf->db_out,"\n");
-  if(retval==0)
+  if(retval==0){
     return RETFAIL;
-
+  }
   return RETOK;
 }
 
@@ -387,6 +405,11 @@ int db_writeline_file(db_line* line,db_config* conf){
       db_writechar(line->filename,conf->db_out,i);
       break;
     }
+    case db_bcount : {
+      db_writeint(line->bcount,conf->db_out,i);
+      break;
+    }
+
     case db_mtime : {
       db_write_time_base64(line->mtime,conf->db_out,i);
       break;
@@ -445,7 +468,44 @@ int db_writeline_file(db_line* line,db_config* conf){
       db_writeoct(line->perm,conf->db_out,i);
       break;
     }
-    
+#ifdef WITH_MHASH
+    case db_crc32 : {
+      db_write_byte_base64(line->crc32,
+			   mhash_get_block_size(MHASH_CRC32),
+			   conf->db_out,i);
+      break;
+    }
+    case db_crc32b : {
+      db_write_byte_base64(line->crc32b,
+			   mhash_get_block_size(MHASH_CRC32B),
+			   conf->db_out,i);
+      break;
+    }
+    case db_haval : {
+      db_write_byte_base64(line->haval,
+			   mhash_get_block_size(MHASH_HAVAL),
+			   conf->db_out,i);
+      break;
+    }
+    case db_rmd128 : {
+      db_write_byte_base64(line->rmd128,
+			   mhash_get_block_size(MHASH_RIPEMD128),
+			   conf->db_out,i);
+      break;
+    }
+    case db_snefru : {
+      db_write_byte_base64(line->snefru,
+			   mhash_get_block_size(MHASH_SNEFRU),
+			   conf->db_out,i);
+      break;
+    }
+    case db_gost : {
+      db_write_byte_base64(line->gost ,
+			   mhash_get_block_size(MHASH_GOST),
+			   conf->db_out,i);
+      break;
+    }
+#endif
     default : {
       error(0,"Not implemented in db_writeline_file %i\n",
 	    conf->db_out_order[i]);
