@@ -1,3 +1,4 @@
+
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999 Rami Lehti,Pablo Virolainen
@@ -25,6 +26,7 @@
 #include <limits.h>
 #include <sys/stat.h>
 #include <errno.h>
+#include <time.h>
 #include "error.h"
 #include "config.h"
 #include "gnu_regex.h"
@@ -32,15 +34,25 @@
 #include "gen_list.h"
 #include "seltree.h"
 #include "db_config.h"
+
+#define CLOCK_SKEW 5
+
+#ifdef WITH_MHASH
+#include <mhash.h>
+#else 
 #include "cipher.h"
+#endif
 
 char* strrxtok(char* rx)
 {
   char*p=NULL;
   int i=0;
-  int lastslash=0;
 
-  for(i=0;i<strlen(rx);i++){
+  /* This assumes that the first character is a slash */
+  int lastslash=1;
+
+  /* i=0 because we want to return at least the first slash */
+  for(i=1;i<strlen(rx);i++){
     switch(rx[i])
       {
       case '/':
@@ -70,16 +82,19 @@ seltree* get_seltree_node(seltree* tree,char* path)
   seltree* node=NULL;
   list* r=NULL;
 
-  if(tree==NULL)
+  if(tree==NULL){
     return NULL;
+  }
 
-  if(strncmp(path,tree->path,strlen(path))==0)
+  if(strncmp(path,tree->path,strlen(path))==0){
     return tree;
+  }
   else{
     for(r=tree->childs;r;r=r->next){
       node=get_seltree_node((seltree*)r->data,path);
-      if(node!=NULL)
+      if(node!=NULL){
 	return node;
+      }
     }
   }
   return NULL;
@@ -99,9 +114,10 @@ seltree* new_seltree_node(seltree* tree,char*path)
   node->checked=0;
   if(tree!=NULL){
     parent=get_seltree_node(tree,strrxtok(path));
-    if(parent==NULL)
+    if(parent==NULL){
       parent=new_seltree_node(tree,strrxtok(path));
-
+    }
+    
     parent->childs=list_append(parent->childs,(void*)node);
     node->parent=parent;
   }else {
@@ -119,17 +135,38 @@ void gen_seltree(list* rxlist,seltree* tree,char type){
   rx_rule* rxc=NULL;
 
   for(r=rxlist;r;r=r->next){
+    char* data;
+    
     rxtok=strrxtok(((rx_rule*)r->data)->rx);
     curnode=get_seltree_node(tree,rxtok);
     if(curnode==NULL){
       curnode=new_seltree_node(tree,rxtok);
     }
+    
+    /* We have to add '^' to the first charaster of string... 
+     *
+     */
+
+    data=(char*)malloc(strlen(((rx_rule*)r->data)->rx)+1+1);
+
+    if (data==NULL){
+      error(0,"Not enough memory for regexpr compile... exiting..\n");
+      abort();
+    }
+    
+    /* FIX ME! (returnvalue) */
+    
+    strcpy(data+1,((rx_rule*)r->data)->rx);
+    
+    data[0]='^';
+    
     rxtmp=(regex_t*)malloc(sizeof(regex_t));
-    if( regcomp(rxtmp,((rx_rule*)r->data)->rx,REG_EXTENDED|REG_NOSUB)){
+    if( regcomp(rxtmp,data,REG_EXTENDED|REG_NOSUB)){
       error(0,"Error in selective regexp:%s",((rx_rule*)r->data)->rx);
+      free(data);
     }else{
       rxc=(rx_rule*)malloc(sizeof(rx_rule));
-      rxc->rx=((rx_rule*)r->data)->rx;
+      rxc->rx=data;
       rxc->crx=rxtmp;
       rxc->attr=((rx_rule*)r->data)->attr;
       switch (type){
@@ -147,6 +184,9 @@ void gen_seltree(list* rxlist,seltree* tree,char type){
       }
       }
     }
+    /* Data should not be free'ed because it's in rxc struct
+     * and freeing is done if error occour.
+     */
   }
 
 
@@ -155,6 +195,7 @@ void gen_seltree(list* rxlist,seltree* tree,char type){
 list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
 {
   db_line* fil=NULL;
+  time_t cur_time;
   struct stat fs;
   int sres=0;
   
@@ -169,54 +210,102 @@ list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
    * symlinks stats not the file that it points to. */
   sres=lstat(fil->filename,&fs);
   if(sres==-1){
-    error(0,"lstat() failed for %s:%s\n",fil->filename,strerror(errno));
+    char* er=strerror(errno);
+    if (er==NULL) {
+      error(0,"lstat() failed for %s. strerror failed for %i\n",fil->filename,errno);
+    } else {
+      error(0,"lstat() failed for %s:%s\n",fil->filename,strerror(errno));
+    }
+    free(fil->filename);
+    free(fil);
     *addok=RETFAIL;
     return listp;
-  }
-  if(DB_INODE&attr)
-    fil->inode=fs.st_ino;
-  else 
-    fil->inode=0;
-
-  if(DB_UID&attr)
-    fil->uid=fs.st_uid;
-  else
-    fil->uid=0;
-
-  if(DB_GID&attr)
-    fil->gid=fs.st_gid;
-  else
-    fil->gid=0;
-
-  if(DB_PERM&attr)
-    fil->perm=fs.st_mode;
-  else
-    fil->perm=0;
-
-  if(DB_SIZE&attr||DB_SIZEG&attr)
-    fil->size=fs.st_size;
-  else 
-    fil->size=0;
-
-  if(DB_LNKCOUNT&attr)
-    fil->nlink=fs.st_nlink;
-  else
-    fil->nlink=0;
-
-  if(DB_MTIME&attr)
-    fil->mtime=fs.st_mtime;
-  else
-    fil->mtime=0;
-
-  if(DB_CTIME&attr)
-    fil->ctime=fs.st_ctime;
-  else
-    fil->ctime=0;
+  } 
   
-  if(DB_ATIME&attr)
+  cur_time=time(NULL);
+
+  if (cur_time==(time_t)-1) {
+    char* er=strerror(errno);
+    if (er==NULL) {
+      error(0,"Can not get current time. strerror failed for %i\n",errno);
+    } else {
+      error(0,"Can not get current time with reason %s\n",er);
+    }
+  } else {
+    
+    if(fs.st_atime>cur_time){
+      error(CLOCK_SKEW,"%s atime in future\n",fil->filename);
+    }
+    if(fs.st_mtime>cur_time){
+      error(CLOCK_SKEW,"%s mtime in future\n",fil->filename);
+    }
+    if(fs.st_ctime>cur_time){
+      error(CLOCK_SKEW,"%s ctime in future\n",fil->filename);
+    }
+  }
+  
+  fil->perm_o=fs.st_mode;
+  fil->size_o=fs.st_size;
+  
+  if(DB_INODE&attr){
+    fil->inode=fs.st_ino;
+  } else {
+    fil->inode=0;
+  }
+
+  if(DB_UID&attr) {
+    fil->uid=fs.st_uid;
+  }else {
+    fil->uid=0;
+  }
+
+  if(DB_GID&attr){
+    fil->gid=fs.st_gid;
+  }else{
+    fil->gid=0;
+  }
+
+  if(DB_PERM&attr){
+    fil->perm=fs.st_mode;
+  }else{
+    fil->perm=0;
+  }
+
+  if(DB_SIZE&attr||DB_SIZEG&attr){
+    fil->size=fs.st_size;
+  }else{
+    fil->size=0;
+  }
+  
+  if(DB_LNKCOUNT&attr){
+    fil->nlink=fs.st_nlink;
+  }else {
+    fil->nlink=0;
+  }
+
+  if(DB_MTIME&attr){
+    fil->mtime=fs.st_mtime;
+  }else{
+    fil->mtime=0;
+  }
+
+  if(DB_CTIME&attr){
+    fil->ctime=fs.st_ctime;
+  }else{
+    fil->ctime=0;
+  }
+  
+  if(DB_ATIME&attr){
     fil->atime=fs.st_atime;
-  else
+  }else{
     fil->atime=0;
+  }
+
+  if(DB_BCOUNT&attr){
+    fil->bcount=fs.st_blocks;
+  } else {
+    fil->bcount=0;
+  }
 
   if(S_ISDIR(fs.st_mode)||S_ISCHR(fs.st_mode)
      ||S_ISBLK(fs.st_mode)||S_ISFIFO(fs.st_mode)
@@ -225,6 +314,14 @@ list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
     fil->sha1=NULL;
     fil->tiger=NULL;
     fil->rmd160=NULL;
+#ifdef WITH_MHASH
+    fil->crc32=NULL;
+    fil->crc32b=NULL;
+    fil->haval=NULL;
+    fil->rmd128=NULL;
+    fil->snefru=NULL;
+    fil->gost=NULL;
+#endif
   }else {
     /* 1 if needs to be set
      * 0 otherwise */
@@ -232,6 +329,14 @@ list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
     fil->sha1=DB_SHA1&attr?"":NULL;
     fil->tiger=DB_TIGER&attr?"":NULL;
     fil->rmd160=DB_RMD160&attr?"":NULL;
+#ifdef WITH_MHASH
+    fil->crc32=DB_CRC32&attr?"":NULL;
+    fil->crc32b=DB_CRC32B&attr?"":NULL;
+    fil->rmd128=DB_RMD128&attr?"":NULL;
+    fil->snefru=DB_SNEFRU&attr?"":NULL;
+    fil->gost=DB_GOST&attr?"":NULL;
+    fil->haval=DB_HAVAL&attr?"":NULL;
+#endif
   }
   listp=list_append(listp,(void*)fil);
 
@@ -269,9 +374,10 @@ int check_node_for_match(seltree*node,char*text,int retval,int* attr)
 {
   int top=0;
   
-  if(node==NULL)
+  if(node==NULL){
     return retval;
-
+  }
+  
   /* We need this to check whether this was the first one *
    * to be called and not a recursive call */
   if(!((retval&16)==16)){
@@ -302,10 +408,11 @@ int check_node_for_match(seltree*node,char*text,int retval,int* attr)
 
   /* Negative regexps are the strongest so they are checked last */
   /* If this file is to be added */
-  if(retval)
-    if(!check_list_for_match(node->neg_rx_lst,text,attr))
+  if(retval){
+    if(!check_list_for_match(node->neg_rx_lst,text,attr)){
       retval=0;
-  
+    }
+  }
   /* Now we discard the info whether a match was made or not *
    * and just return 0,1 or 2 */
   if(top){
@@ -326,11 +433,9 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
   char* fullname=NULL;
   int e=0;
   int matchattr=attr;
-  long td=0;
+  long td=-1;
   int addok=RETOK;
 
-  if( strncmp(tree->path,"",strlen(tree->path))==0)
-    tree->path="/";
   if(!(dirh=opendir(tree->path))){
     error(5,"traverse_tree():%s: %s\n", strerror(errno),tree->path);
     return file_lst;
@@ -383,14 +488,20 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
       if(addfile){
 	file_lst=add_file_to_list(file_lst,fullname,matchattr,&addok);
 	if(addfile!=2 && addok!=RETFAIL){
-	  if(S_ISDIR(((db_line*)file_lst->tail->data)->perm)){
-	    a=get_seltree_node(tree,((db_line*)file_lst->tail->data)->filename);
-	    if(a==NULL)
-	      a=new_seltree_node(tree,((db_line*)file_lst->tail->data)->filename);
+	  if(S_ISDIR(((db_line*)file_lst->header->tail->data)->perm_o)){
+	    a=get_seltree_node(tree,
+			       ((db_line*)file_lst->header->tail->data)
+			       ->filename);
+	    if(a==NULL){
+	      a=new_seltree_node(tree,
+				 ((db_line*)file_lst->header->tail->data)
+				 ->filename);
+	    }
 	    file_lst=traverse_tree(a,file_lst,attr);
 	  }
 	}
       } else {
+	error(230,"File %s does not match\n",fullname);
 	free(fullname);
       }
     } else{ /* This dir is not added by default */
@@ -401,18 +512,21 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
       if(addfile){
 	file_lst=add_file_to_list(file_lst,fullname,matchattr,&addok);
 	if(addfile!=2 && addok!=RETFAIL){
-	  if(S_ISDIR(((db_line*)file_lst->tail->data)->perm)){
+	  if(S_ISDIR(((db_line*)file_lst->header->tail->data)->perm_o)){
 	    a=get_seltree_node(tree,
-			       ((db_line*)file_lst->tail->data)->filename);
+			       ((db_line*)file_lst->header->tail->data)
+			       ->filename);
 	    if(a==NULL)
 	      a=new_seltree_node(tree,
-				 ((db_line*)file_lst->tail->data)->filename);
+				 ((db_line*)file_lst->header->tail->data)
+				 ->filename);
 	    
 	    file_lst=traverse_tree(a,file_lst,
 				   matchattr);
 	  }
 	}
       } else {
+	error(230,"File %s does not match\n",fullname);
 	free(fullname);
       }
       
@@ -446,7 +560,7 @@ list* gen_list(list* prxlist,list* nrxlist,list* erxlist)
   list* r=NULL;
   seltree* tree=NULL;
 
-  tree=new_seltree_node(NULL,"");
+  tree=new_seltree_node(NULL,"/");
 
   gen_seltree(prxlist,tree,'s');
   gen_seltree(nrxlist,tree,'n');
